/*
 * StreamFeaturesetNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class FeaturesetNode extends AbstractPacket implements StreamFeatureset {
    //Constructors
    public FeaturesetNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected FeaturesetNode(StreamElement parent, FeaturesetNode base) {
        super(parent, base);
    }

    //Methods
    public StreamBuilder createBuilder() {
        return new AbstractPacketBuilder(this) {
            protected StreamBuilder.Status processNestedElementStart(StreamParseState s) {
                StreamDataFactory   sdf = s.getContext().getDataFactory();
                NSI                 name = s.getName();
                
                setExtendedData(sdf.createElementBuilder(name, StreamFeature.class));
                
                return EXTENDED_DATA;
            }
        };
    }
    
    public PacketError getError() {
        return null;
    }
    public void setError(PacketError err) throws IllegalArgumentException {
    }
    
    public List listFeatures() {
        return listElements(StreamFeature.class);
    }
    
    public StreamFeature getFeature(String ns) throws IllegalArgumentException {
        if (!Utilities.isValidString(ns))
            throw new IllegalArgumentException("namespace cannot be null or \"\"");
        
        return (StreamFeature)getFirstElement(null, ns, StreamFeature.class);
    }
    public StreamFeature getFeature(Class iface) throws IllegalArgumentException {
        if (iface == null)
            throw new IllegalArgumentException("interface cannot be null");
        if (!StreamFeature.class.isAssignableFrom(iface))
            throw new IllegalArgumentException("interface must be derived from StreamFeature");
        
        return (StreamFeature)getFirstElement(iface);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new FeaturesetNode(parent, this);
    }
}