/*
 * JSO.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;
import java.util.Map;

import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.JSOImplementation;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamElementFactory;
import org.jabberstudio.jso.io.DOMExporter;
import org.jabberstudio.jso.io.DOMImporter;
import org.jabberstudio.jso.io.Exporter;
import org.jabberstudio.jso.io.Importer;
import org.jabberstudio.jso.io.XMLExporter;
import org.jabberstudio.jso.io.XMLImporter;
import org.jabberstudio.jso.util.Service;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.io.DOMReader;
import net.outer_planes.jso.io.DOMWriter;
import net.outer_planes.jso.io.XMLReader;
import net.outer_planes.jso.io.XMLWriter;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class JSO extends JSOImplementation {
    //Fields
    private DataFactory     _SDF;
    
    private Service         _Exporters;
    private Service         _Importers;
    
    /** Creates a new instance of JSO */
    public JSO() {
        Service                 svc = new Service(StreamElementFactory.class, null);
        Iterator                found = svc.getProviders().keySet().iterator();
        StreamElementFactory    sdf;
        
        _SDF = new DataFactory();
        
        //Setup "built-ins"
        //getDataFactory().registerElementFactory(new StandardPacketFactory());
        //getDataFactory().registerElementFactory(new CoreExtensionFactory());
        getDataFactory().registerElementFactory(new UnknownFactory());
        
        //Setup SPI-providers
        while (found.hasNext()) {
            try {
                sdf = (StreamElementFactory)svc.obtainProvider((String)found.next());
                getDataFactory().registerElementFactory(sdf);
            } catch (Exception e) {
            }
        }
    }

    //Methods
    public StreamDataFactory getDataFactory() {
        return _SDF;
    }
    /** <p>
     * Creates a new <tt>StreamDataFactory</tt>, with the registered
     * factories.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The preconfigured <tt>StreamDataFactory</tt>.
     */
    public StreamDataFactory createDataFactory() {
        return new DataFactory((DataFactory)getDataFactory());
    }
    public StreamDataFactory createDataFactory(String ns) {
        DataFactory     df = new DataFactory(ns, (DataFactory)getDataFactory());
        
        return df;
    }
    
    public void registerElementFactory(StreamElementFactory sef) {
        getDataFactory().registerElementFactory(sef);
    }
    public void registerElementFactory(NSI name, StreamElementFactory sef) {
        getDataFactory().registerElementFactory(name, sef);
    }
    public void unregisterElementFactory(StreamElementFactory sef) {
        getDataFactory().unregisterElementFactory(sef);
    }
    public void unregisterElementFactory(NSI name, StreamElementFactory sef) {
        getDataFactory().unregisterElementFactory(name, sef);
    }
    
    public Map getNamespaceMappings() {
        return getDataFactory().getNamespaceMappings();
    }
    public void setNamespaceMappings(Map nss) {
        getDataFactory().setNamespaceMappings(nss);
    }
    public void putNamespaceMapping(String prefix, String uri) {
        getDataFactory().putNamespaceMapping(prefix, uri);
    }
    public void removeNamespaceMapping(String prefix) {
        getDataFactory().removeNamespaceMapping(prefix);
    }
    
    public XPath createXPath(String expr) throws SAXPathException {
        return getDataFactory().createXPath(expr);
    }
    public XPath createXPath(String expr, Map nss) throws SAXPathException {
        return getDataFactory().createXPath(expr, nss);
    }
    
 
    public StreamContext createStreamContext() {
        return createStreamContext(null, null);
    }
    public StreamContext createStreamContext(NSI name, String prefix) throws IllegalArgumentException {
        BasicStreamContext  bsc;
        StreamContext       ctx;
        
        //Validate parameters
        if (name == null)
            name = BasicStreamContext.NAME;
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local name cannot be null or \"\"");
        if (name.getNamespaceURI() == null)
            name = new NSI(name.getLocalName(), Utilities.STREAM_NAMESPACE);
        if (prefix == null)
            prefix = Utilities.STREAM_PREFIX;

        try {
            ctx = (StreamContext)getDataFactory().createElementNode(name, StreamContext.class);
            ctx.addNamespace(prefix, ctx.getNamespaceURI());
        } catch (IllegalArgumentException iae) {
            bsc = new BasicStreamContext(name, prefix);
            bsc.setDataFactory(createDataFactory());
            
            ctx = bsc;
        }
        
        return ctx;
    }
    
    public Stream createStream(String ns) throws IllegalArgumentException {
        return createStream(ns, null, null);
    }
    public Stream createStream(String ns, StreamContext in, StreamContext out) throws IllegalArgumentException {
        //Validate input
        if (ns == null)
            ns = "";
        if (in == null)
            in = createStreamContext();
        if (out == null)
            out = createStreamContext();

        if (ns.equals(Utilities.COMPONENT_ACCEPT_NAMESPACE) || ns.equals(Utilities.COMPONENT_CONNECT_NAMESPACE))
            return new ComponentStream(this, (DataFactory)createDataFactory(ns), ns, in, out);
        return new BasicStream(this, (DataFactory)createDataFactory(ns), ns, in, out);
    }
    
    private synchronized Service getExporterService() {
        if (_Exporters == null) {
            _Exporters = new Service(Exporter.class, null);
        }
        
        return _Exporters;
    }
    public Exporter createExporter(String type, StreamElement ctx) throws IllegalArgumentException, RuntimeException {
        Exporter        exp = (Exporter)getExporterService().obtainProvider(type);
        
        exp.configure(ctx);
        
        return exp;
    }
    public DOMExporter createDOMExporter(StreamElement ctx) throws IllegalArgumentException {
        DOMExporter     exp;
        
        try {
            exp = (DOMExporter)createExporter(DOMExporter.class.getName(), ctx);
        } catch (IllegalArgumentException iae) {
            exp = new DOMWriter(ctx);
        }
        
        return exp;
    }
    public XMLExporter createXMLExporter(StreamElement ctx) throws IllegalArgumentException {
        XMLExporter     exp;
        
        try {
            exp = (XMLExporter)createExporter(XMLExporter.class.getName(), ctx);
        } catch (IllegalArgumentException iae) {
            exp = new XMLWriter(ctx);
        }
        
        return exp;
    }
    
    private synchronized Service getImporterService() {
        if (_Importers == null) {
            _Importers = new Service(Importer.class, null);
        }
        
        return _Importers;
    }
    public Importer createImporter(String type, StreamElement ctx) throws IllegalArgumentException, RuntimeException {
        Importer        imp = (Importer)getImporterService().obtainProvider(type);
        
        imp.configure(ctx);
        
        return imp;
    }
    public DOMImporter createDOMImporter(StreamElement ctx) throws IllegalArgumentException {
        DOMImporter     imp;
        
        try {
            imp = (DOMImporter)createImporter(DOMImporter.class.getName(), ctx);
        } catch (IllegalArgumentException iae) {
            imp = new DOMReader(ctx);
        }
        
        return imp;
    }
    public XMLImporter createXMLImporter(StreamElement ctx) throws IllegalArgumentException {
        XMLImporter     imp;
        
        try {
            imp = (XMLImporter)createImporter(XMLImporter.class.getName(), ctx);
        } catch (IllegalArgumentException iae) {
            imp = new XMLReader(ctx);
        }
        
        return imp;
    }
}