/*
 * NamespaceNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamObjectVisitor;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class NamespaceNode extends AbstractObject implements StreamNamespace {
    //Fields
    private StreamElement   _Parent;
    private String          _Prefix;
    private String          _URI;
    
    /** Creates a new instance of NamespaceNode */
    public NamespaceNode(StreamElement parent, String prefix, String uri) {
        setParent(parent);
        setPrefix(prefix);
        setURI(uri);
    }
    
    //Methods
    public String getPrefix() {
        return _Prefix;
    }
    private void setPrefix(String prefix) {
        if (prefix == null)
            prefix = "";
        _Prefix = prefix;
    }
    
    public String getURI() {
        return _URI;
    }
    private void setURI(String uri) {
        _URI = uri;
    }
    
    public StreamElement getParent() {
        return _Parent;
    }
    public void setParent(StreamElement p) throws IllegalStateException {
        _Parent = p;
    }
    
    public StreamDataFactory getDataFactory() {
        StreamElement   parent = getParent();
        return (parent != null) ? parent.getDataFactory() : null;
    }
    
    public void accept(StreamObjectVisitor v) {
    }
    
    public StreamObject copy(StreamElement p) {
        return new NamespaceNode(p, getPrefix(), getURI());
    }
    
    public void detach() {
        StreamElement   parent = getParent();
        
        if (parent != null) {
            parent.removeNamespace(getPrefix(), getURI());
            setParent(null);
        }
    }
}