/*
 * StringAttribute.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class StringAttribute extends AbstractAttribute implements StreamAttribute {
    //Fields
    private NSI         _Name;
    private String      _Value;

    //Constructors
    /** Creates a new instance of AttributeNode */
    public StringAttribute(NSI name) {
        this(null, name, null);
    }
    public StringAttribute(StreamElement p, NSI name, String val) {
        super(p);
        setNSI(name);
        setValue(val);
    }
    
    //Methods
    public StreamObject copy(StreamElement p) {
        return new StringAttribute(p, getNSI(), getValue());
    }
    
    public NSI getNSI() {
        return _Name;
    }
    private void setNSI(NSI name) {
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local name cannot be null");
        if (name.getNamespaceURI() == null)
            name = new NSI(name.getLocalName(), "");
        _Name = name;
    }
    
    public String getValue() {
        return _Value;
    }
    public void setValue(String val) throws IllegalArgumentException {
        if (val == null)
            val = "";
        _Value = val;
    }
}