/*
 * UnknownPacketNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class UnknownPacketNode extends AbstractPacket implements Packet {
    //Inner-classes
    public static class StrType extends Packet.Type {
        //Constructors
        public StrType(String name) throws IllegalArgumentException {
            super(name);
        }
    }
    
    //Constructors
    /** Creates a new instance of UnknownPacketNode */
    public UnknownPacketNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    public UnknownPacketNode(StreamElement parent, UnknownPacketNode base) {
        super(parent, base);
    }
    
    //Methods
    public StreamBuilder createBuilder() {
        return new ElementBuilder(this);
    }
    
    protected Packet.Type determineType(String name) {
        Packet.Type     type = super.determineType(name);
        
        if (type == null)
            type = new StrType(name);
        
        return type;
    }
    
    public PacketError getError() {
        Iterator    itr = listElements("error", getNamespaceURI()).iterator();
        
        return (PacketError)(itr.hasNext() ? itr.next() : null);
    }
    public void setError(PacketError err) {
        Iterator    itr = listElements("error", getNamespaceURI()).iterator();
        
        //Remove existing
        while (itr.hasNext()) {
            ((StreamElement)itr.next()).detach();
        }

        configurePacketError(err);
        super.add(err);
    }
    protected void configurePacketError(PacketError err) {
        if (err == null) {
            setType(null);
        } else {
            setType(ERROR);
        }
    }
    
    public void add(StreamNode node) throws UnsupportedOperationException {
        if          (node == null) {
            return;
        } else {
            if (node instanceof PacketError) {
                configurePacketError((PacketError)node);
            }
            super.add(node);
        }
    }
    public void insert(int idx, StreamNode node) throws UnsupportedOperationException {
        if          (node == null) {
            return;
        } else {
            if (node instanceof PacketError) {
                configurePacketError((PacketError)node);
            }
            super.insert(idx, node);
        }
    }
    public void remove(StreamNode node) throws UnsupportedOperationException {
        if          (node == null) {
            return;
        } else {
            if (node instanceof PacketError) {
                configurePacketError(null);
            }
            super.remove(node);
        }
    }

    /** <p>
     * Creates an "almost-exact" duplicate of this <code>StreamNode</code>,
     * using the given parent <code>StreamElement</code>.  By "almost-exact",
     * all information for this <code>StreamNode</code>, and its contained
     * objects (obtained via <code>{@link #listNodes}</code>) are duplicated.
     * The parent for this <code>StreamNode</code> is one specified by
     * <code>p</code>, rather than thatreferenced by this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case no
     * parent is attached to the newly created <code>StreamNode</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamNode</code>.
     */
    public StreamObject copy(StreamElement p) {
        return new UnknownPacketNode(p, this);
    }
}