/*
 * NamespaceManager.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.ixp;


import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class NamespaceManager {
    //Fields
    private NamespaceManager    _Parent;
    private Map                 _NSS;
    
    //Constructors
    public NamespaceManager(NamespaceManager parent, Map nss) {
        _Parent = parent;
        _NSS = (nss != null) ? nss : java.util.Collections.EMPTY_MAP;
    }
    
    //Methods
    public NamespaceManager getParent() {
        return _Parent;
    }
    public NamespaceManager getManager(int depth) {
        return (depth == getDepth()) ? this : (getParent() != null) ? getParent().getManager(depth) : null;
    }

    public int getDepth() {
        return ((getParent() != null) ? getParent().getDepth() : -1) + 1;
    }

    public Map getNamespaces() {
        return _NSS;
    }
    public void setNamespaces(Map nss) {
        _NSS.clear();
        _NSS.putAll(nss);
    }
    
    public Map getNamespaces(int depth) {
        return (depth == getDepth()) ? _NSS : (getParent() != null) ? getParent().getNamespaces(depth) : null;
    }
    public Set getNamespaceURIs(int depth) {
        Map     mapping = getNamespaces(depth);
        Set     uris = new java.util.TreeSet();
        
        if (mapping != null)
            uris.addAll(mapping.values());
        
        return uris;
    }
    public Set getNamespacePrefixes(int depth) {
        Map     mapping = getNamespaces(depth);
        Set     prefixes = new java.util.TreeSet();
        
        if (mapping != null)
            prefixes.addAll(mapping.keySet());
        
        return prefixes;
    }

    public String getNamespaceURI(String prefix) {
        String      uri = null;
        Iterator    itr = _NSS.entrySet().iterator();

        if (prefix == null)
            prefix = "";
        
        if (Utilities.equateStrings(Utilities.XML_PREFIX, prefix))
            uri = Utilities.XML_NAMESPACE;

        while ((uri == null) && itr.hasNext()) {
            Map.Entry   entry = (Map.Entry)itr.next();

            if (entry.getKey().equals(prefix))
                uri = (String)entry.getValue();
        }

        return (uri != null) ? uri : ((getParent() != null) ? getParent().getNamespaceURI(prefix) : null);
    }
    public String getNamespacePrefix(String uri) {
        String      prefix = null;
        Iterator    itr = _NSS.entrySet().iterator();

        if (uri == null)
            uri = "";
        
        if (Utilities.equateStrings(Utilities.XML_NAMESPACE, uri))
            prefix = Utilities.XML_PREFIX;

        while ((prefix == null) && itr.hasNext()) {
            Map.Entry   entry = (Map.Entry)itr.next();

            if (entry.getValue().equals(uri))
                prefix = (String)entry.getKey();
        }

        return (prefix != null) ? prefix : ((getParent() != null) ? getParent().getNamespacePrefix(uri) : null);
    }
    
    
    public String toString() {
        return "namespace manager (depth=" + getDepth() + ";nss=" + getNamespaces() + ")";
    }
}