/*
 * ClientMechanism.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.sasl.SASLAuthPacket;
import org.jabberstudio.jso.sasl.SASLClientInfo;
import org.jabberstudio.jso.sasl.SASLFailedException;
import org.jabberstudio.jso.sasl.SASLFailurePacket;
import org.jabberstudio.jso.sasl.SASLPacket;

/**
 *
 * @author  mmiller
 * @version $VERSION$
 * @since   JSO-0.12
 */
public abstract class ClientMechanism extends AbstractMechanism {
    //Fields
    private JID _AuthZid;
    
    //Constructors
    public ClientMechanism(SASLClientInfo ci) {
        super(ci.getDataFactory(), ci.getServer(), ci.getCallbackHandler(), ci.getProperties());
        _AuthZid = ci.getAuthorizationID();
    }
    
    //Methods
    public JID getAuthorizationID() {
        return _AuthZid;
    }
    
    public SASLPacket evaluate(SASLPacket input) throws IllegalArgumentException, IllegalStateException, SASLFailedException {
        Status              current = getCurrentStatus();
        SASLPacket.Action   act = (input != null) ? input.getAction() : null;
        SASLPacket          output = null;
        
        if (current == READY) {
            if      (input == null)
                output = generateAuth();
            else if (act == SASLPacket.CHALLENGE)
                output = evaluateChallenge(input);
            else
                throw new IllegalStateException("mechanism is not in a compatible state");
        } else if   (current.isAuthenticating()) {
            if      (act == SASLPacket.SUCCESS)
                completeSuccess(input);
            else if (act == SASLPacket.FAILURE)
                throw completeFailure((SASLFailurePacket)input);
            else if (act == SASLPacket.CHALLENGE)
                output = evaluateChallenge(input);
            else
                throw new IllegalStateException("mechanism is not in a compatible state");
        } else {
            throw new IllegalStateException("mechanism is already complete");
        }
        
        return output;
    }
    
    protected SASLPacket generateAuth() {
        SASLAuthPacket  auth = (SASLAuthPacket)getDataFactory().createPacketNode(SASLAuthPacket.NAME, SASLAuthPacket.class);
        
        auth.setMechanismName(getMechanismName());
        
        return auth;
    }
    protected abstract SASLPacket evaluateChallenge(SASLPacket challenge) throws IllegalStateException, SASLFailedException;
    
    protected void completeSuccess(SASLPacket success) {
        setCurrentStatus(SUCCEEDED);
   }
    protected SASLFailedException completeFailure(SASLFailurePacket failed) {
        setCurrentStatus(FAILED);
        
        return new SASLFailedException(failed);
    }
}