/*
 * SASLPacketNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import java.util.Map;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.sasl.SASLPacket;
import org.jabberstudio.jso.util.ByteCodec;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.AbstractPacket;
import net.outer_planes.jso.AbstractPacketBuilder;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class SASLPacketNode extends AbstractPacket implements SASLPacket {
    //"Constants"
    public static final Map     SASL_NAMES;
    
    static {
        Map     temp = new java.util.TreeMap(NSI.STRICT_COMPARATOR);
        
        temp.put(NAME_ABORT, ABORT);
        temp.put(NAME_AUTH, AUTHENTICATE);
        temp.put(NAME_CHALLENGE, CHALLENGE);
        temp.put(NAME_FAILURE, FAILURE);
        temp.put(NAME_RESPONSE, RESPONSE);
        temp.put(NAME_SUCCESS, SUCCESS);
        
        SASL_NAMES = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Fields
    private Action      _Act;
    
    //Constructors
    public SASLPacketNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    protected SASLPacketNode(StreamElement parent, SASLPacketNode base) {
        super(parent, base);
    }

    //Methods
    public StreamBuilder createBuilder() {
        return new AbstractPacketBuilder(this);
    }
    
    public SASLPacket.Action getAction() {
        if (_Act == null) {
            _Act = (Action)SASL_NAMES.get(getNSI());
        }
        
        return _Act;
    }
    
    public byte[] getData() {
        ByteCodec   b64 = ByteCodec.BASE64;
        String      enc = normalizeTrimText();
        
        return (Utilities.isValidString(enc) ? b64.decode(enc) : new byte[0]);
    }
    public void setData(byte[] data) {
        ByteCodec   b64 = ByteCodec.BASE64;
        
        if (data == null)
            data = new byte[0];
        
        clearText();
        addText(b64.encode(data));
    }

    public PacketError getError() {
        return null;
    }
    public void setError(PacketError err) throws IllegalArgumentException {
    }

    public StreamObject copy(StreamElement parent) {
        return new SASLPacketNode(parent, this);
    }
}