package net.outer_planes.jso.sasl.mechanisms;


import java.io.*;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import org.jabberstudio.jso.sasl.SASLPacket;
import org.jabberstudio.jso.util.Utilities;

public class MD5MessageCodec
{
    private static final String  _token = "[^\"()<>@,;:\\<>/\\[\\]?=\\{\\} \t]+";
    private static final Pattern _pattern;
    
    static {
        _pattern = Pattern.compile("(" + _token + ")=(?:(" + _token + ")|(?:\"([^\"]+)(?:\")))");
    }

    public static Properties decode(byte[] b64) {
        Properties  props = new Properties();
        String      challenge;
        Matcher     m;
        String      name;
        String      val;

        if ((b64 == null) || (b64.length == 0)) {
            return props;
        }
        
        try {
            challenge = new String(b64, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new IllegalStateException("UTF-8 is not a supported encoding");
        }

        m = _pattern.matcher(challenge);
        while (m.find()) {
            name = m.group(1);
            val = (((val = m.group(2)) != null) ? val : m.group(3));
            if ((name != null) && (val != null)) {
                props.setProperty(name, val);
            }
        }
        
        return props;
    }
    
    public static byte[] encode(Properties props) {
        Iterator        itr;
        Map.Entry       entry;
        String          key, value;
        StringBuffer    buffer = new StringBuffer();
        byte[]          b64 = null;
        
        if (props == null)
            props = new Properties();

        buffer = appendParameter(buffer, props, "charset", false);
        buffer = appendParameter(buffer, props, "username", true);
        buffer = appendParameter(buffer, props, "realm", true);
        buffer = appendParameter(buffer, props, "nonce", true);
        buffer = appendParameter(buffer, props, "cnonce", true);
        buffer = appendParameter(buffer, props, "nc", false);
        buffer = appendParameter(buffer, props, "qop", false);
        buffer = appendParameter(buffer, props, "digest-uri", true);
        buffer = appendParameter(buffer, props, "response", false);
        
        try {
            b64 = buffer.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException uee) {
            
        }

        return b64;
    }
    private static final StringBuffer appendParameter(StringBuffer buffer, Properties props, String key, boolean quoted) {
        String      value;
        
        if (buffer == null)
            buffer = new StringBuffer();
        if (key == null)
            return buffer;
        
        value = props.getProperty(key);
        if (value == null)
            value = "";
        if (buffer.length() > 0)
            buffer.append(",");
        
        buffer.append(key).append("=");
        if (quoted)
            buffer.append("\"");
        buffer.append(value);
        if (quoted)
            buffer.append("\"");
        
        return buffer;
    }
}