/*
 * BindQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.JIDFormatException;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.util.stringprep.ResourceprepProfile;
import org.jabberstudio.jso.x.core.BindQuery;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class BindQueryNode extends ExtensionNode implements BindQuery {
    //"Constants"
    public static final NSI     ELEMNAME_RESOURCE   = new NSI("resource", NAMESPACE);
    public static final NSI     ELEMNAME_JID        = new NSI("jid", NAMESPACE);
    
    //Constructors
    public BindQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected BindQueryNode(StreamElement parent, BindQueryNode base) {
        super(parent, base);
    }

    //Methods
    public String getResource() {
        StreamElement   elem = getFirstElement(ELEMNAME_RESOURCE);
        
        return ((elem != null) ? elem.normalizeTrimText() : "");
    }
    public void setResource(String res) throws IllegalArgumentException {
        StreamElement   elem = getFirstElement(ELEMNAME_RESOURCE);
        
        if          (Utilities.isValidString(res)) {
            res = ResourceprepProfile.getInstance().prepare(res);
            
            if (elem == null)
                elem = addElement(ELEMNAME_RESOURCE);
            elem.clearText();
            elem.addText(res);
        } else if   (elem != null) {
            elem.detach();
        }
    }
    
    
    public JID getJID() {
        StreamElement   elem = getFirstElement(ELEMNAME_JID);
        JID             jid = null;
        
        try {
            if (elem != null)
                jid = getDataFactory().createJID(elem.normalizeTrimText());
        } catch (JIDFormatException jfe) {
        }
        
        return jid;
    }
    public void setJID(JID jid) {
        StreamElement   elem = getFirstElement(ELEMNAME_JID);
        
        if          (jid != null) {
            if (elem == null)
                elem = addElement(ELEMNAME_JID);
            elem.clearText();
            elem.addText(jid.toString());
        } else if   (elem != null) {
            elem.detach();
        }
    }
    
    public StreamObject copy(StreamElement parent) {
        return new BindQueryNode(parent, this);
    }
}