/*
 * PrivacyItemNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;


import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.JIDFormatException;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.core.PrivacyItem;
import org.jabberstudio.jso.x.core.PrivacyQuery;
import org.jabberstudio.jso.x.core.RosterItem;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class PrivacyItemNode extends ElementNode implements PrivacyItem {
    //"Constants"
    public static final NSI     ATTRNAME_ACTION = new NSI("action", null);
    public static final NSI     ATTRNAME_TYPE   = new NSI("type", null);
    public static final NSI     ATTRNAME_ORDER  = new NSI("order", null);
    public static final NSI     ATTRNAME_VALUE  = new NSI("value", null);

    public static final NSI     ELEMNAME_IQ             = new NSI("iq", PrivacyQuery.NAMESPACE);
    public static final NSI     ELEMNAME_MESSAGE        = new NSI("message", PrivacyQuery.NAMESPACE);
    public static final NSI     ELEMNAME_PRESENCE_IN    = new NSI("presence-in", PrivacyQuery.NAMESPACE);
    public static final NSI     ELEMNAME_PRESENCE_OUT   = new NSI("presence-out", PrivacyQuery.NAMESPACE);

    //Constructors
    public PrivacyItemNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected PrivacyItemNode(StreamElement parent, PrivacyItemNode base) {
        super(parent, base);
    }

    //Methods
    public PrivacyItem.Action getAction() {
        Object      val = getAttributeObject(ATTRNAME_ACTION);
        Action      act = null;

        if          (val instanceof Action) {
            act = (Action)val;
        } else if   (val != null) {
            String  temp = val.toString();

            if      (Utilities.equateStrings(temp, ALLOW.toString()))
                act = ALLOW;
            else if (Utilities.equateStrings(temp, DENY.toString()))
                act = DENY;

            if (act != null)
                setAttributeObject(ATTRNAME_ACTION, act);
        }

        return act;
    }
    public void setAction(PrivacyItem.Action act) {
        setAttributeObject(ATTRNAME_ACTION, act);
    }

    public PrivacyItem.Type getType() {
        Object      val = getAttributeObject(ATTRNAME_TYPE);
        Type        type = null;

        if          (val instanceof Type) {
            type = (Type)val;
        } else if   (val != null) {
            String  temp = val.toString();

            if      (Utilities.equateStrings(temp, JID.toString()))
                type = JID;
            else if (Utilities.equateStrings(temp, GROUP.toString()))
                type = GROUP;
            else if (Utilities.equateStrings(temp, SUBSCRIPTION.toString()))
                type = SUBSCRIPTION;

            if (type != null)
                setAttributeObject(ATTRNAME_TYPE, type);
        }

        return type;
    }
    public void setType(PrivacyItem.Type type) {
        setAttributeObject(ATTRNAME_TYPE, type);
    }

    public int getOrder() {
        Object  val = getAttributeObject(ATTRNAME_ORDER);
        int     order = 0;

        if          (val instanceof Number) {
            order = ((Number)val).intValue();
        } else if   (val != null) {
            Number  temp = null;

            try {
                temp = Integer.valueOf(val.toString());
                order = temp.intValue();
                setAttributeObject(ATTRNAME_ORDER, temp);
            } catch (NumberFormatException nfe) {
            }
        }

        return order;
    }
    public void setOrder(int order) throws IllegalArgumentException {
        if (order < 0)
            throw new IllegalArgumentException("order cannot be less than 0");

        setAttributeObject(ATTRNAME_ORDER, new Integer(order));
    }

    public String getValue() {
        String      val = getAttributeValue(ATTRNAME_VALUE);

        return ((val != null) ? val : "");
    }
    public void setValue(String val) {
        setAttributeValue(ATTRNAME_VALUE, Utilities.isValidString(val) ? val : null);
    }

    public JID getJIDValue() throws IllegalStateException {
        Object  val = getAttributeObject(ATTRNAME_VALUE);
        JID     jid = null;

        if (getType() != JID)
            throw new IllegalStateException("Type must be JID");

        if          (val instanceof JID) {
            jid = (JID)val;
        } else if   (val != null) {
            try {
                jid = new JID(val);
                setAttributeObject(ATTRNAME_VALUE, jid);
            } catch (JIDFormatException jfe) {
            }
        }

        return jid;
    }
    public void setJIDValue(JID val) {
        setType(JID);
        setAttributeObject(ATTRNAME_VALUE, val);
    }

    public String getGroupValue() throws IllegalStateException {
        if (getType() != GROUP)
            throw new IllegalStateException("Type must be GROUP");

        return getValue();
    }
    public void setGroupValue(String val) {
        setType(GROUP);
        setValue(val);
    }

    public RosterItem.SubscriptionType getSubscriptionValue() throws IllegalStateException {
        Object                      val = getAttributeObject(ATTRNAME_VALUE);
        RosterItem.SubscriptionType type = null;

        if (getType() != SUBSCRIPTION)
            throw new IllegalStateException("Type must be SUBSCRIPTION");

        if          (val instanceof RosterItem.SubscriptionType) {
            type = (RosterItem.SubscriptionType)val;
        } else if   (val != null) {
            String      temp = val.toString();

            if      (Utilities.equateStrings(temp, RosterItem.BOTH.toString()))
                type = RosterItem.BOTH;
            else if (Utilities.equateStrings(temp, RosterItem.FROM.toString()))
                type = RosterItem.FROM;
            else if (Utilities.equateStrings(temp, RosterItem.NONE.toString()))
                type = RosterItem.NONE;
            else if (Utilities.equateStrings(temp, RosterItem.TO.toString()))
                type = RosterItem.TO;

            if (type != null)
                setAttributeObject(ATTRNAME_VALUE, type);
        }

        return type;
    }
    public void setSubscriptionValue(RosterItem.SubscriptionType val) {
        setType(SUBSCRIPTION);
        setAttributeObject(ATTRNAME_VALUE, val);
    }

    public boolean isAppliedToIQ() {
        return (getFirstElement(ELEMNAME_IQ) != null);
    }
    public void setAppliedToIQ(boolean block) {
        StreamElement   elem = getFirstElement(ELEMNAME_IQ);

        if      (block && (elem == null))
            elem = addElement(ELEMNAME_IQ);
        else if (!block && (elem != null))
            elem.detach();
    }

    public boolean isAppliedToMessage() {
        return (getFirstElement(ELEMNAME_MESSAGE) != null);
    }
    public void setAppliedToMessage(boolean block) {
        StreamElement   elem = getFirstElement(ELEMNAME_MESSAGE);

        if      (block && (elem == null))
            elem = addElement(ELEMNAME_MESSAGE);
        else if (!block && (elem != null))
            elem.detach();
    }

    public boolean isAppliedToPresenceIn() {
        return (getFirstElement(ELEMNAME_PRESENCE_IN) != null);
    }
    public void setAppliedToPresenceIn(boolean block) {
        StreamElement   elem = getFirstElement(ELEMNAME_PRESENCE_IN);

        if      (block && (elem == null))
            elem = addElement(ELEMNAME_PRESENCE_IN);
        else if (!block && (elem != null))
            elem.detach();
    }

    public boolean isAppliedToPresenceOut() {
        return (getFirstElement(ELEMNAME_PRESENCE_OUT) != null);
    }
    public void setAppliedToPresenceOut(boolean block) {
        StreamElement   elem = getFirstElement(ELEMNAME_PRESENCE_OUT);

        if      (block && (elem == null))
            elem = addElement(ELEMNAME_PRESENCE_OUT);
        else if (!block && (elem != null))
            elem.detach();
    }

    public StreamObject copy(StreamElement parent) {
        return new PrivacyItemNode(parent, this);
    }
}