/*
 * ExpireExtensionNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import java.util.Date;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.x.info.ExpireExtension;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 */
public class ExpireExtensionNode extends ExtensionNode implements ExpireExtension {
    //Inner-classes
    private static class StoredWrapper {
        //Fields
        private Date    _Stored;
        
        //Constructors
        public StoredWrapper(Date stored) {
            _Stored = stored;
        }
        
        //Methods
        public final Date getDate() {
            return _Stored;
        }
        public final String toString() {
            return String.valueOf(getDate().getTime());
        }
    }
    
    //"Constants"
    public static final Long    DEFAULT_SECS = new Long(0);
    
    public static final NSI     ATTRNAME_SECONDS    = new NSI("seconds", null);
    public static final NSI     ATTRNAME_STORED     = new NSI("stored", null);
    
    //Constructors
    public ExpireExtensionNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected ExpireExtensionNode(StreamElement parent, ExpireExtensionNode base) {
        super(parent, base);
    }
    
    //Methods
    public long getSeconds() {
        Object      val = getAttributeObject(ATTRNAME_SECONDS);
        Number      seconds = null;
        
        if (val instanceof Number) {
            seconds = (Number)val;
        } else if   (val != null) {
            try {
                seconds = Long.valueOf(val.toString());
                setAttributeObject(ATTRNAME_SECONDS, seconds);
            } catch (NumberFormatException nfe) {
            }
        }
        
        if (seconds == null)
            seconds = DEFAULT_SECS;
        
        return seconds.longValue();
    }
    public void setSeconds(long secs) {
        Number  val = ((secs < 0) ? null : new Long(secs));
        
        setAttributeObject(ATTRNAME_SECONDS, new Long(secs));
    }

    public Date getStored() {
        Object      val = getAttributeObject(ATTRNAME_STORED);
        Number      temp;
        Date        stored = null;
        
        if (val instanceof StoredWrapper) {
            stored = ((StoredWrapper)val).getDate();
        } else if   (val != null) {
            try {
                temp = Long.valueOf(val.toString());
                stored = new Date(temp.longValue());
                setStored(stored);
            } catch (NumberFormatException nfe) {
            }
        }
        
        return stored;
    }
    public void setStored(Date stored) {
        StoredWrapper   wrap = null;
        
        if (stored != null)
            wrap = new StoredWrapper(stored);
        setAttributeObject(ATTRNAME_STORED, wrap);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new ExpireExtensionNode(parent, this);
    }
}