/*
 * OOBExtensionNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import java.net.MalformedURLException;
import java.net.URL;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.info.OutOfBandExtension;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 */
public class OOBExtensionNode extends ExtensionNode implements OutOfBandExtension {
    //Fields
    public static final String  URL_NAME    = "url";
    public static final String  DESC_NAME   = "desc";
    
    //Constructors
    protected OOBExtensionNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    protected OOBExtensionNode(StreamElement parent, OOBExtensionNode base) {
        super(parent, base);
    }
    
    //Methods
    public static OOBExtensionNode createOutOfBandIQ(StreamDataFactory sdf) {
        return new OOBExtensionNode(sdf,  IQ_NAME);
    }
    public static OOBExtensionNode createOutOfBandX(StreamDataFactory sdf) {
        return new OOBExtensionNode(sdf,  X_NAME);
    }
    
    public URL getURL() {
        URL             url = null;
        StreamElement   elem = getFirstElement(URL_NAME, getNamespaceURI());
        
        if (elem != null) {
            try {   url = new URL(elem.normalizeTrimText()); } catch (MalformedURLException murle) {}
        }
        
        return url;
    }
    public void setURL(URL url) {
        StreamElement   elem = getFirstElement(URL_NAME, getNamespaceURI());
        
        if (elem != null)
            elem.detach();
        
        if (url != null)
            addElement(URL_NAME).addText(url.toString());
    }
    
    public String getDescription() {
        StreamElement   elem = getFirstElement(DESC_NAME, getNamespaceURI());
        
        return ((elem != null) ? elem.normalizeTrimText() : "");
    }
    public void setDescription(String desc) {
        StreamElement   elem = getFirstElement(DESC_NAME, getNamespaceURI());
        
        if          (Utilities.isValidString(desc)) {
            if (elem != null) {
                elem.clearText();
            } else {
                elem = addElement(DESC_NAME);
            }
            elem.addText(desc);
        } else if   (elem != null) {
            elem.detach();
        }
        if (elem != null)
            elem.detach();
        
        if (Utilities.isValidString(desc))
            addElement(DESC_NAME).addText(desc);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new OOBExtensionNode(parent, this);
    }
}
