/*
 * TimeQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.info.TimeQuery;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 */
public class TimeQueryNode extends ExtensionNode implements TimeQuery {
    //"Constants"
    public static final SimpleDateFormat    UTC_FORMAT  = new SimpleDateFormat("yyyyMMdd'T'HH:mm:ss");
    
    public static final NSI                 NSI_UTC     = new NSI("utc", NAMESPACE);
    public static final NSI                 NSI_TZ      = new NSI("tz", NAMESPACE);
    public static final NSI                 NSI_DISPLAY = new NSI("display", NAMESPACE);
    
    static {
        UTC_FORMAT.setTimeZone(TimeZone.getTimeZone("GMT-00:00"));
    }
    
    //Constructors
    public TimeQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    public TimeQueryNode(StreamElement parent, TimeQueryNode base) {
        super(parent, base);
    }
    
    //Methods
    public Date getTime() {
        StreamElement   elem = getFirstElement(NSI_UTC);
        String          utc = ((elem != null) ? elem.normalizeTrimText() : "");
        Date            time = null;
        
        try {
            time = UTC_FORMAT.parse(utc);
        } catch (ParseException pe) {
        }
        
        return time;
    }
    public void setTime(Date time) {
        StreamElement   elem = getFirstElement(NSI_UTC);
        String          utc;

        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //...and in with the New
        if (time != null) {
            utc = UTC_FORMAT.format(time);
            addElement(NSI_UTC).addText(utc);
        }
    }
    
    public String getTimezone() {
        StreamElement   elem = getFirstElement(NSI_TZ);
        String          tz = ((elem != null) ? elem.normalizeTrimText() : "");
        
        return tz;
    }
    public void setTimezone(String tz) {
        StreamElement   elem = getFirstElement(NSI_TZ);
        
        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //..and in with the New
        if (Utilities.isValidString(tz))
            addElement(NSI_TZ).addText(tz);
    }
    
    public String getDisplay() {
        StreamElement   elem = getFirstElement(NSI_DISPLAY);
        String          display = ((elem != null) ? elem.normalizeTrimText() : "");
        
        return display;
    }
    public void setDisplay(String display) {
        StreamElement   elem = getFirstElement(NSI_DISPLAY);
        
        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //...and in with the New
        if (Utilities.isValidString(display))
            addElement(NSI_DISPLAY).addText(display);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new TimeQueryNode(parent, this);
    }
}