/*
 * VersionQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.info.VersionQuery;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 */
public class VersionQueryNode extends ExtensionNode implements VersionQuery {
    //"Constants"
    public static final NSI     NSI_NAME =  new NSI("name", NAMESPACE);
    public static final NSI     NSI_OS      = new NSI("os", NAMESPACE);
    public static final NSI     NSI_VERSION = new NSI("version", NAMESPACE);
    
    //Constructors
    public VersionQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    public VersionQueryNode(StreamElement parent, VersionQueryNode base) {
        super(parent, base);
    }
    
    //Methods
    public String getName() {
        StreamElement   elem = getFirstElement(NSI_NAME);
        String          name = ((elem != null) ? elem.normalizeTrimText() : "");
        
        return name;
    }
    public void setName(String name) {
        StreamElement   elem = getFirstElement(NSI_NAME);
        
        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //...and in with the New
        if (Utilities.isValidString(name))
            addElement(NSI_NAME).addText(name);
    }
    
    public String getOS() {
        StreamElement   elem = getFirstElement(NSI_OS);
        String          os = ((elem != null) ? elem.normalizeTrimText() : "");
        
        return os;
    }
    public void setOS(String os) {
        StreamElement   elem = getFirstElement(NSI_OS);
        
        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //...and in with the New
        if (Utilities.isValidString(os))
            addElement(NSI_OS).addText(os);
    }
    
    public String getVersion() {
        StreamElement   elem = getFirstElement(NSI_VERSION);
        String          ver = ((elem != null) ? elem.normalizeTrimText() : "");
        
        return ver;
    }
    public void setVersion(String ver) {
        StreamElement   elem = getFirstElement(NSI_VERSION);
        
        //Out with the old...
        if (elem != null)
            elem.detach();
        
        //...and in with the New
        if (Utilities.isValidString(ver))
            addElement(NSI_VERSION).addText(ver);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new VersionQueryNode(parent, this);
    }
}