/*
 * JSOXPath.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package net.outer_planes.jso.xpath;

import java.util.Iterator;
import java.util.Map;

import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.xpath.*;

import org.jaxen.*;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class JSOXPath extends org.jaxen.BaseXPath {
    //"Constants"
    private static Map  XPATH_FUNCTIONS;
    
    static {
        Map                 funcs = new java.util.TreeMap();
        AbstractFunction    func;
        
        func = new IsJidFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new JidNodeFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new JidDomainFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new JidResourceFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new MatchJidFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new EqualsJidFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new IsClassFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new ImplsClassFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new LocaleFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new EqualsLocaleFunction();
        funcs.put(func.getFunctionName(), func);
        
        func = new MatchLocaleFunction();
        funcs.put(func.getFunctionName(), func);
        
        XPATH_FUNCTIONS = java.util.Collections.unmodifiableMap(funcs);
    }
    
    //Constructors
    public JSOXPath(String xpath) throws org.jaxen.JaxenException {
        this(xpath, new StreamNavigator());
    }
    public JSOXPath(String xpath, StreamNavigator nav) throws org.jaxen.JaxenException {
        super(xpath, nav);
        
        SimpleFunctionContext   fCtx = (SimpleFunctionContext)getFunctionContext();
        Iterator                fItr = XPATH_FUNCTIONS.values().iterator();
        
        //Setup "default" functions
        while (fItr.hasNext()) {
            ((AbstractFunction)fItr.next()).register(null, fCtx);
        }
        
        setFunctionContext(getFunctionContext());
        setVariableContext(getVariableContext());
    }
    
    //Methods
    public void setupNamespaces(StreamElement context) throws JaxenException {
        StreamElement   parent;
        StreamNamespace nsObj;
        Iterator        nsItr;
        String          nsURI;
        String          nsPrefix;
        
        if (context != null) {
            parent = context.getParent();
            
            //Setup based on base's parent first
            if (parent != null)
                setupNamespaces(parent);
            
            //Setup based on base
            nsItr = context.getDeclaredNamespaces().iterator();
            while (nsItr.hasNext()) {
                nsObj = (StreamNamespace)nsItr.next();
                nsURI = nsObj.getURI();
                nsPrefix = nsObj.getPrefix();
                
                addNamespace(nsPrefix, nsURI);
            }
        }
    }
}
