/*
 * StreamNavigator.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package net.outer_planes.jso.xpath;

import org.jaxen.DefaultNavigator;


import java.util.Iterator;

import org.jaxen.XPath;

import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamText;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class StreamNavigator extends DefaultNavigator {
    //Fields
    
    //Constructors
    public StreamNavigator() {
    }
    
    //Methods
    public boolean isText(Object obj) {
        return (obj instanceof StreamText);
    }
    
    public boolean isProcessingInstruction(Object obj) {
        return false;
    }
    
    public boolean isNamespace(Object obj) {
        return (obj instanceof StreamNamespace);
    }
    
    public boolean isElement(Object obj) {
        return (obj instanceof StreamElement);
    }
    
    public boolean isDocument(Object obj) {
        return (obj instanceof StreamContext);
    }
    
    public boolean isComment(Object obj) {
        return false;
    }
    
    public boolean isAttribute(Object obj) {
        return (obj instanceof StreamAttribute);
    }
    
    public String getAttributeStringValue(Object obj) {
        return ((StreamAttribute)obj).getValue();
    }
    
    public String getAttributeQName(Object obj) {
        return ((StreamAttribute)obj).getQualifiedName();
    }
    
    public String getAttributeNamespaceUri(Object obj) {
        return ((StreamAttribute)obj).getNamespaceURI();
    }
    
    public String getAttributeName(Object obj) {
        return ((StreamAttribute)obj).getLocalName();
    }
    
    public String getElementStringValue(Object obj) {
        StreamElement   elem = (StreamElement)obj;
        Iterator        nodeItr = elem.listNodes().iterator();
        StringBuffer    buffer = new StringBuffer();
        
        while (nodeItr.hasNext()) {
            StreamNode      node = (StreamNode)nodeItr.next();
            
            if (node instanceof StreamText) {
                buffer.append(((StreamText)node).getValue());
            } else if   (node instanceof StreamElement) {
                buffer.append(getElementStringValue(node));
            }
        }
        
        return buffer.toString();
    }
    
    public String getElementQName(Object obj) {
        return ((StreamElement)obj).getQualifiedName();
    }
    
    public String getElementNamespaceUri(Object obj) {
        return ((StreamElement)obj).getNamespaceURI();
    }
    
    public String getElementName(Object obj) {
        return ((StreamElement)obj).getLocalName();
    }
    
    public String getTextStringValue(Object obj) {
        return ((StreamText)obj).getValue();
    }
    
    public String getCommentStringValue(Object obj) {
        return null;
    }
    
    public String getNamespaceStringValue(Object obj) {
        return ((StreamNamespace)obj).getURI();
    }
    
    public String getNamespacePrefix(Object obj) {
        return ((StreamNamespace)obj).getPrefix();
    }
    
    public Object getDocumentNode(Object obj) {
        StreamContext   ctx = null;

        if (obj instanceof StreamObject) {
            ctx = ((StreamObject)obj).getContext();
        }
        
        return ctx;
    }
    
    
    public XPath parseXPath(String xpath) throws org.saxpath.SAXPathException {
        return new JSOXPath(xpath, this);
    }
    
    public Iterator getParentAxisIterator(Object obj) throws org.jaxen.UnsupportedAxisException {
        StreamElement   parent = null;
        Iterator        itr = null;
        
        if (obj instanceof StreamObject) {
            parent = ((StreamObject)obj).getParent();
            itr = new org.jaxen.util.SingleObjectIterator(parent);
        }
        
        return itr;
    }
    
    public Iterator getChildAxisIterator(Object obj) throws org.jaxen.UnsupportedAxisException {
        Iterator    itr = null;
        
        if (obj instanceof StreamNode) {
            itr = ((StreamNode)obj).listNodes().iterator();
        }
        
        return itr;
    }
    
    public Iterator getAttributeAxisIterator(Object obj) throws org.jaxen.UnsupportedAxisException {
        Iterator    itr = null;
        
        if (obj instanceof StreamElement) {
            itr = ((StreamElement)obj).getAttributes().values().iterator();
        }
        
        return itr;
    }
    
    public Iterator getNamespaceAxisIterator(Object obj) throws org.jaxen.UnsupportedAxisException {
        Iterator    itr = null;
        
        if (obj instanceof StreamElement) {
            itr = ((StreamElement)obj).getDeclaredNamespaces().iterator();
        }
        
        return itr;
    }
    
    public String translateNamespacePrefixToUri(String prefix, Object obj) {
        StreamElement   elem = null;
        String          uri = null;

        if (obj instanceof StreamAttribute) {
            elem = ((StreamAttribute)obj).getParent();
        }
        if (obj instanceof StreamElement) {
            elem = (StreamElement)obj;
        }
        
        if (elem != null) {
            uri = elem.getNamespaceURI(prefix);
        }
        
        return uri;
    }
}