/*
 * StreamNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;
import java.util.Locale;


/**
 * <p>
 * Basic data unit for any stream-based "component" information.  This
 * interface and its immediate derivations can be used to construct the
 * XML-formatted stream data.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamNode extends StreamObject {
    //Methods
    /**
     * <p>
     * Retrieves the language governing this <tt>StreamNode</tt>. This method
     * first tries to determine this value locally, then searches its parent
     * if it cannot be determined.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. This value
     * may be "" if the language cannot be determined.</p>
     * 
     * @return The language governing this <tt>StreamObject</tt>.
     * @since   JSO-0.10
     * @deprecated As of JSO-0.12, this method is replaced with {@link
     * #getLocale()}.
     */
    public String getLanguage();
    /**
     * <p>
     * Retrieves the locale for this <tt>StreamNode</tt>. This method
     * first tries to determine this value locally, then searches its parent
     * if it cannot be determined.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If the value
     * cannot be determined, the platform default locale is returned.</p>
     * 
     * @return The locale for this <tt>StreamObject</tt>.
     * @since   JSO-0.12
     */
    public Locale getLocale();
    
    /**
     * <p>
     * Lists the <code>StreamNode</code> objects that are contained by this
     * <code>StreamNode</code>.  This list may or may not be modifiable.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The list of contained <code>StreamNode</code> objects.
     * @since   JSO-0.2
     */
    public List listNodes();
}