/*
 * StreamText.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;

/**
 * <p>
 * A string of character data from the stream.  This interface defines the
 * basis for representing data similar to the XML DOM's <code>Text</code> or
 * <code>CDATA</code> node types.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamText extends StreamNode {
    //Methods
    /**
     * <p>
     * Retreives the String value for this <code>StreamText</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return the String for this <code>StreamText</code>.
     * @since   JSO-0.2
     */
    public String getValue();
    /**
     * <p>
     * Sets the String value for this <code>StreamText</code>.</p>
     *
     * <p>
     * The value of <code>val</code> cannot be <code>null</code> or "",
     * otherwise an <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param val the new String value for this <code>StreamText</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public void setValue(String val) throws IllegalArgumentException;
}