/*
 * StreamStatusDispatcher.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;

/**
 * <p>
 * "Registration" interface for objects that dispatch
 * <tt>StreamStatusEvent</tt>. This interface provides the methods to register
 * and unregister <tt>StreamStatusListener</tt> objects.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.6
 */
public interface StreamStatusDispatcher {
    //Methods
    /**
     * <p>
     * Registers the given <tt>StreamStatusListener</tt> for any changes in the
     * <tt>StreamStatus</tt>.</p>
     *
     * <p>
     * The value of <tt>l</tt> may be <tt>null</tt>, in which case no actions
     * are performed. No effort is made to verify if <tt>l</tt> is already
     * registered.</p>
     *
     * @param l The <tt>StreamStatusListener</tt> to register.
     * @since   JSO-0.12
     */
    public void addStreamStatusListener(StreamStatusListener l);
    /**
     * <p>
     * Unregisters the given <tt>StreamStatusListener</tt> for any changes in
     * the <tt>StreamStatus</tt>.</p>
     *
     * <p>
     * The value of <tt>l</tt> may be <tt>null</tt>, in which case no actions
     * are performed. No effort is made to verify if <tt>l</tt> is registered
     * more than once; the first occurance found is removed.</p>
     *
     * @param l The <tt>StreamStatusListener</tt> to unregister.
     * @since   JSO-0.12
     */
    public void removeStreamStatusListener(StreamStatusListener l);
}