/*
 * SASLAuthPacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Primary interface for representing the packets for SASL interaction.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface SASLAuthPacket extends SASLPacket {
    //"Constants"
    /** Qualified name for <tt>SASLAuthPacket</tt>
     * ({urn:ietf:params:xml:ns:xmpp-sasl}auth).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME    = NAME_AUTH;
    
    //Methods
    /**
     * <p>
     * Retrieves the mechanism name for this <tt>SASLAuthPacket</tt>. This is
     * the name used to determine the mechanism to evaluate with.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The mechanism name.
     * @since   JSO-0.12
     */
    public String getMechanismName();
    /**
     * <p>
     * Sets the mechanism name for this <tt>SASLAuthPacket</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed.</p>
     *
     * @param name The new mechanism name.
     * @since   JSO-0.12
     */
    public void setMechanismName(String name);
}