/*
 * RealmInputCallback.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl.callback;


import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Underlying security services instantiate and pass a
 * <tt>RealmInputCallback</tt> to the <tt>handle</tt> method of a
 * <tt>CallbackHandler</tt> to retrieve realm information.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class RealmInputCallback extends javax.security.auth.callback.TextInputCallback {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>RealmCallback</tt> with the given prompt, and no
     * default realm.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param prompt The prompt used to request the realm.
     * @throws IllegalArgumentException If the paramter is invalid.
     * @since   JSO-0.12
     */
    public RealmInputCallback(String prompt) throws IllegalArgumentException {
        super(prompt);
    }
    /**
     * <p>
     * Creates a new <tt>RealmCallback</tt> with the given prompt and default
     * realm.</p>
     *
     * <p>
     * The value of <tt>prompt</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>realm</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param prompt The prompt used to request the realm.
     * @param realm The default realm.
     * @throws IllegalArgumentException If the paramter is invalid.
     * @since   JSO-0.12
     */
    public RealmInputCallback(String prompt, String realm) throws IllegalArgumentException {
        super(prompt, realm);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the default realm.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> or "".</p>
     *
     * @return The default realm.
     * @since   JSO-0.12
     */
    public String getDefaultRealm() {
        return getDefaultText();
    }
    
    /**
     * <p>
     * Retreives the realm.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> or "".</p>
     *
     * @return The realm.
     * @since   JSO-.12
     */
    public String getRealm() {
        return getText();
    }
    /**
     * <p>
     * Sets the realm.</p>
     *
     * <p>
     * The value of <tt>realm</tt> may be <tt>null</tt> or "".</p>
     *
     * @param realm The new realm.
     * @since   JSO-0.12
     */
    public void setRealm(String realm) {
        setText(realm);
    }
}