/*
 * StartTLSFeature.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.tls;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamFeature;

/**
 * <p>
 * <tt>StreamFeature</tt> for STARTTLS support.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface StartTLSFeature extends StreamFeature {
    //"Constants"
    /** The namespace URI for <tt>StartTLSFeature</tt>
     * (urn:ietf:params:xml:ns:xmpp-tls).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "urn:ietf:params:xml:ns:xmpp-tls";
    /** The qualified name for <tt>StartTLSFeature</tt>
     * ({urn:ietf:params:xml:ns:xmpp-tls}starttls).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("starttls", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Determines if this feature is required to use.</p>
     *
     * @return <tt>true</tt> if STARTTLS negotiation is required.
     * @since   JSO-0.12
     */
    public boolean isRequired();
    /**
     * <p>
     * Sets this feature as required.</p>
     *
     * @param req The new required flag.
     * @since   JSO-0.12
     */
    public void setRequired(boolean req);
}