/*
 * RosterQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:iq:roster" information.</p>
 *
 * <p>
 * Note that any modifications to the data contained by an instance of
 * <code>RosterQuery</code> is not automatically reflected back to the
 * "server" or "component".  The <code>RosterQuery</code> (wrapped in an
 * appropriate <code>InfoQuery</code>) must still be sent for changes to have
 * any real effect.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public interface RosterQuery extends Extension {
    //"Constants"
    /**
     * Namespace governing <code>RosterQuery</code>
     * @since   JSO-0.4
     */
    public static final String  NAMESPACE = "jabber:iq:roster";
    /**
     * Qualified name for <tt>RosterQuery</tt> ({jabber:iq:roster}query).
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the <code>RosterItem</code> for the given <code>JID</code>.</p>
     *
     * <p>
     * This method returns <code>null</code> if there is no
     * <code>RosterItem</code> currently available for <code>jid</code>.</p>
     *
     * <p>
     * The value of <Code>jid</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param jid The <code>JID</code> to search for.
     * @return The <code>RosterItem</code> for <code>jid</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.4
     */
    public RosterItem getItem(JID jid);
    /**
     * <p>
     * Creates a <code>RosterItem</code> for the given <code>JID</code>.
     * If a <code>RosterItem</code> already exists for the given
     * <code>JID</code>, that object is returned, and a new object is not
     * created.</p>
     *
     * <p>
     * The created <code>RosterItem</code> is immediately added to this
     * <code>RosterQuery</code>.  It is not necessary to  call {@link #add}.</p>
     *
     * <p>
     * This method never returns <code>null</code>.</p>
     * @since   JSO-0.4
     */
    public RosterItem createItem(JID jid);
}